# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.pagination import SyncChunkPagination, AsyncChunkPagination
from scale_gp.types.knowledge_bases import ChunkListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChunks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        chunk = client.knowledge_bases.chunks.list(
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(SyncChunkPagination[ChunkListResponse], chunk, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        chunk = client.knowledge_bases.chunks.list(
            knowledge_base_id="knowledge_base_id",
            chunk_id="chunk_id",
            max_chunks=1,
            metadata_filters="metadata_filters",
        )
        assert_matches_type(SyncChunkPagination[ChunkListResponse], chunk, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.knowledge_bases.chunks.with_raw_response.list(
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chunk = response.parse()
        assert_matches_type(SyncChunkPagination[ChunkListResponse], chunk, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.knowledge_bases.chunks.with_streaming_response.list(
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chunk = response.parse()
            assert_matches_type(SyncChunkPagination[ChunkListResponse], chunk, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.chunks.with_raw_response.list(
                knowledge_base_id="",
            )


class TestAsyncChunks:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        chunk = await async_client.knowledge_bases.chunks.list(
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(AsyncChunkPagination[ChunkListResponse], chunk, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        chunk = await async_client.knowledge_bases.chunks.list(
            knowledge_base_id="knowledge_base_id",
            chunk_id="chunk_id",
            max_chunks=1,
            metadata_filters="metadata_filters",
        )
        assert_matches_type(AsyncChunkPagination[ChunkListResponse], chunk, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.chunks.with_raw_response.list(
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chunk = await response.parse()
        assert_matches_type(AsyncChunkPagination[ChunkListResponse], chunk, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.chunks.with_streaming_response.list(
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chunk = await response.parse()
            assert_matches_type(AsyncChunkPagination[ChunkListResponse], chunk, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.chunks.with_raw_response.list(
                knowledge_base_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .chunk_extra_info_schema import ChunkExtraInfoSchema
from .string_extra_info_schema import StringExtraInfoSchema

__all__ = ["ResultSchemaGeneration", "GenerationExtraInfo"]

GenerationExtraInfo: TypeAlias = Annotated[
    Union[ChunkExtraInfoSchema, StringExtraInfoSchema], PropertyInfo(discriminator="schema_type")
]


class ResultSchemaGeneration(BaseModel):
    generation_output: str

    generation_extra_info: Optional[GenerationExtraInfo] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Required, TypeAlias, TypedDict

from ..s3_data_source_config_param import S3DataSourceConfigParam
from ..slack_data_source_config_param import SlackDataSourceConfigParam
from ..local_chunks_source_config_param import LocalChunksSourceConfigParam
from ..s3_data_source_auth_config_param import S3DataSourceAuthConfigParam
from ..confluence_data_source_config_param import ConfluenceDataSourceConfigParam
from ..slack_data_source_auth_config_param import SlackDataSourceAuthConfigParam
from ..share_point_data_source_config_param import SharePointDataSourceConfigParam
from ..token_chunking_strategy_config_param import TokenChunkingStrategyConfigParam
from ..custom_chunking_strategy_config_param import CustomChunkingStrategyConfigParam
from ..google_drive_data_source_config_param import GoogleDriveDataSourceConfigParam
from ..character_chunking_strategy_config_param import CharacterChunkingStrategyConfigParam
from ..confluence_data_source_auth_config_param import ConfluenceDataSourceAuthConfigParam
from ..share_point_data_source_auth_config_param import SharePointDataSourceAuthConfigParam
from ..google_drive_data_source_auth_config_param import GoogleDriveDataSourceAuthConfigParam
from ..azure_blob_storage_data_source_config_param import AzureBlobStorageDataSourceConfigParam
from ..azure_blob_storage_data_source_auth_config_param import AzureBlobStorageDataSourceAuthConfigParam

__all__ = [
    "UploadCreateParams",
    "CreateKnowledgeBaseV2UploadFromDataSourceRequest",
    "CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceConfig",
    "CreateKnowledgeBaseV2UploadFromDataSourceRequestChunkingStrategyConfig",
    "CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceAuthConfig",
    "CreateKnowledgeBaseV2UploadFromLocalChunksRequest",
    "CreateKnowledgeBaseV2UploadFromLocalChunksRequestChunk",
    "CreateKnowledgeBaseV2UploadFromDataSourceIDRequest",
    "CreateKnowledgeBaseV2UploadFromDataSourceIDRequestChunkingStrategyConfig",
]


class CreateKnowledgeBaseV2UploadFromDataSourceRequest(TypedDict, total=False):
    data_source_config: Required[CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceConfig]
    """Configuration for the data source which describes where to find the data."""

    chunking_strategy_config: CreateKnowledgeBaseV2UploadFromDataSourceRequestChunkingStrategyConfig
    """Configuration for the chunking strategy which describes how to chunk the data."""

    data_source_auth_config: CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceAuthConfig
    """
    Configuration for the data source which describes how to authenticate to the
    data source.
    """

    force_reupload: bool
    """Force reingest, regardless the change of the source file."""


CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceConfig: TypeAlias = Union[
    S3DataSourceConfigParam,
    SharePointDataSourceConfigParam,
    GoogleDriveDataSourceConfigParam,
    AzureBlobStorageDataSourceConfigParam,
    ConfluenceDataSourceConfigParam,
    SlackDataSourceConfigParam,
]

CreateKnowledgeBaseV2UploadFromDataSourceRequestChunkingStrategyConfig: TypeAlias = Union[
    CharacterChunkingStrategyConfigParam, TokenChunkingStrategyConfigParam, CustomChunkingStrategyConfigParam
]

CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceAuthConfig: TypeAlias = Union[
    SharePointDataSourceAuthConfigParam,
    AzureBlobStorageDataSourceAuthConfigParam,
    GoogleDriveDataSourceAuthConfigParam,
    S3DataSourceAuthConfigParam,
    ConfluenceDataSourceAuthConfigParam,
    SlackDataSourceAuthConfigParam,
]


class CreateKnowledgeBaseV2UploadFromLocalChunksRequest(TypedDict, total=False):
    data_source_config: Required[LocalChunksSourceConfigParam]
    """Configuration for the data source which describes where to find the data."""

    chunks: Iterable[CreateKnowledgeBaseV2UploadFromLocalChunksRequestChunk]
    """List of chunks."""

    force_reupload: bool
    """Force reingest, regardless the change of the source file."""


class CreateKnowledgeBaseV2UploadFromLocalChunksRequestChunk(TypedDict, total=False):
    chunk_position: Required[int]
    """Position of the chunk in the artifact."""

    text: Required[str]
    """Associated text of the chunk."""

    metadata: object
    """Additional metadata associated with the chunk."""


class CreateKnowledgeBaseV2UploadFromDataSourceIDRequest(TypedDict, total=False):
    chunking_strategy_config: Required[CreateKnowledgeBaseV2UploadFromDataSourceIDRequestChunkingStrategyConfig]
    """Configuration for the chunking strategy which describes how to chunk the data."""

    data_source_id: Required[str]
    """Id of the data source to fetch."""

    force_reupload: bool
    """Force reingest, regardless the change of the source file."""


CreateKnowledgeBaseV2UploadFromDataSourceIDRequestChunkingStrategyConfig: TypeAlias = Union[
    CharacterChunkingStrategyConfigParam, TokenChunkingStrategyConfigParam, CustomChunkingStrategyConfigParam
]

UploadCreateParams: TypeAlias = Union[
    CreateKnowledgeBaseV2UploadFromDataSourceRequest,
    CreateKnowledgeBaseV2UploadFromLocalChunksRequest,
    CreateKnowledgeBaseV2UploadFromDataSourceIDRequest,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import InteractionCreateResponse
from scale_gp._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestInteractions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        interaction = client.interactions.create(
            application_variant_id="application_variant_id",
            input={"query": "query"},
            output={"response": "response"},
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(InteractionCreateResponse, interaction, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        interaction = client.interactions.create(
            application_variant_id="application_variant_id",
            input={"query": "query"},
            output={
                "response": "response",
                "context": [
                    {
                        "text": "text",
                        "score": 0,
                    },
                    {
                        "text": "text",
                        "score": 0,
                    },
                    {
                        "text": "text",
                        "score": 0,
                    },
                ],
            },
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            duration_ms=0,
            operation_metadata={},
            operation_status="SUCCESS",
            thread_id="thread_id",
            trace_spans=[
                {
                    "node_id": "node_id",
                    "operation_type": "COMPLETION",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "duration_ms": 0,
                    "operation_input": {},
                    "operation_metadata": {},
                    "operation_output": {},
                    "operation_status": "SUCCESS",
                },
                {
                    "node_id": "node_id",
                    "operation_type": "COMPLETION",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "duration_ms": 0,
                    "operation_input": {},
                    "operation_metadata": {},
                    "operation_output": {},
                    "operation_status": "SUCCESS",
                },
                {
                    "node_id": "node_id",
                    "operation_type": "COMPLETION",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "duration_ms": 0,
                    "operation_input": {},
                    "operation_metadata": {},
                    "operation_output": {},
                    "operation_status": "SUCCESS",
                },
            ],
        )
        assert_matches_type(InteractionCreateResponse, interaction, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.interactions.with_raw_response.create(
            application_variant_id="application_variant_id",
            input={"query": "query"},
            output={"response": "response"},
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        interaction = response.parse()
        assert_matches_type(InteractionCreateResponse, interaction, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.interactions.with_streaming_response.create(
            application_variant_id="application_variant_id",
            input={"query": "query"},
            output={"response": "response"},
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            interaction = response.parse()
            assert_matches_type(InteractionCreateResponse, interaction, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncInteractions:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        interaction = await async_client.interactions.create(
            application_variant_id="application_variant_id",
            input={"query": "query"},
            output={"response": "response"},
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(InteractionCreateResponse, interaction, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        interaction = await async_client.interactions.create(
            application_variant_id="application_variant_id",
            input={"query": "query"},
            output={
                "response": "response",
                "context": [
                    {
                        "text": "text",
                        "score": 0,
                    },
                    {
                        "text": "text",
                        "score": 0,
                    },
                    {
                        "text": "text",
                        "score": 0,
                    },
                ],
            },
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            duration_ms=0,
            operation_metadata={},
            operation_status="SUCCESS",
            thread_id="thread_id",
            trace_spans=[
                {
                    "node_id": "node_id",
                    "operation_type": "COMPLETION",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "duration_ms": 0,
                    "operation_input": {},
                    "operation_metadata": {},
                    "operation_output": {},
                    "operation_status": "SUCCESS",
                },
                {
                    "node_id": "node_id",
                    "operation_type": "COMPLETION",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "duration_ms": 0,
                    "operation_input": {},
                    "operation_metadata": {},
                    "operation_output": {},
                    "operation_status": "SUCCESS",
                },
                {
                    "node_id": "node_id",
                    "operation_type": "COMPLETION",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "duration_ms": 0,
                    "operation_input": {},
                    "operation_metadata": {},
                    "operation_output": {},
                    "operation_status": "SUCCESS",
                },
            ],
        )
        assert_matches_type(InteractionCreateResponse, interaction, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.interactions.with_raw_response.create(
            application_variant_id="application_variant_id",
            input={"query": "query"},
            output={"response": "response"},
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        interaction = await response.parse()
        assert_matches_type(InteractionCreateResponse, interaction, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.interactions.with_streaming_response.create(
            application_variant_id="application_variant_id",
            input={"query": "query"},
            output={"response": "response"},
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            interaction = await response.parse()
            assert_matches_type(InteractionCreateResponse, interaction, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from .test_cases import (
    TestCasesResource,
    AsyncTestCasesResource,
    TestCasesResourceWithRawResponse,
    AsyncTestCasesResourceWithRawResponse,
    TestCasesResourceWithStreamingResponse,
    AsyncTestCasesResourceWithStreamingResponse,
)

__all__ = [
    "HistoryResource",
    "AsyncHistoryResource",
    "HistoryResourceWithRawResponse",
    "AsyncHistoryResourceWithRawResponse",
    "HistoryResourceWithStreamingResponse",
    "AsyncHistoryResourceWithStreamingResponse",
    "TestCasesResource",
    "AsyncTestCasesResource",
    "TestCasesResourceWithRawResponse",
    "AsyncTestCasesResourceWithRawResponse",
    "TestCasesResourceWithStreamingResponse",
    "AsyncTestCasesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, overload
from typing_extensions import Literal

import httpx

from ..types import completion_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._streaming import Stream, AsyncStream
from .._base_client import make_request_options
from ..types.completions_response import CompletionsResponse
from ..types.shared.completion_response import CompletionResponse

__all__ = ["CompletionsResource", "AsyncCompletionsResource"]


class CompletionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return CompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return CompletionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        model: Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ],
        prompt: str,
        account_id: str | NotGiven = NOT_GIVEN,
        images: Iterable[completion_create_params.Image] | NotGiven = NOT_GIVEN,
        model_parameters: completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        stream: Literal[False] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionsResponse:
        """
        ### Description

        Given a user's input, runs LLM inference to produce the model's response.

        ### Details

        LLM [completions](https://scale-egp.readme.io/docs/completions-1) have many use
        cases, such as content summarization, question-answering, and text generation.

        The `model` parameter determines which LLM will be used to generate the
        completion. Keep in mind that different models have varying sizes, costs, and
        may perform differently across different tasks.

        The user input, commonly referred to as the "prompt", is a required field in the
        request body. The quality of the model's response can vary greatly depending on
        the input prompt. Good prompt engineering can significantly enhance the response
        quality. If you encounter suboptimal results, consider writing more specific
        instructions or providing examples to the LLM before trying more expensive
        techniques such as swapping in other models or finetuning.

        By default, the endpoint will return the entire response as one whole object. If
        you would prefer to stream the completion in real-time, you can achieve this by
        setting the `stream` flag to `true`.

        Args:
          model: The ID of the model to use for completions.

              Users have two options:

              - Option 1: Use one of the supported models from the dropdown.
              - Option 2: Enter the ID of a custom model.

              Note: For custom models we currently only support models finetuned using using
              the Scale-hosted LLM-Engine API.

          prompt: Prompt for which to generate the completion.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing a more
              specific prompt here before trying more expensive techniques such as swapping in
              other models or finetuning the underlying LLM.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          images: List of image urls to be used for image based completions. Leave empty for text
              based completions.

          model_parameters: Configuration parameters for the completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ],
        prompt: str,
        stream: Literal[True],
        account_id: str | NotGiven = NOT_GIVEN,
        images: Iterable[completion_create_params.Image] | NotGiven = NOT_GIVEN,
        model_parameters: completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Stream[CompletionResponse]:
        """
        ### Description

        Given a user's input, runs LLM inference to produce the model's response.

        ### Details

        LLM [completions](https://scale-egp.readme.io/docs/completions-1) have many use
        cases, such as content summarization, question-answering, and text generation.

        The `model` parameter determines which LLM will be used to generate the
        completion. Keep in mind that different models have varying sizes, costs, and
        may perform differently across different tasks.

        The user input, commonly referred to as the "prompt", is a required field in the
        request body. The quality of the model's response can vary greatly depending on
        the input prompt. Good prompt engineering can significantly enhance the response
        quality. If you encounter suboptimal results, consider writing more specific
        instructions or providing examples to the LLM before trying more expensive
        techniques such as swapping in other models or finetuning.

        By default, the endpoint will return the entire response as one whole object. If
        you would prefer to stream the completion in real-time, you can achieve this by
        setting the `stream` flag to `true`.

        Args:
          model: The ID of the model to use for completions.

              Users have two options:

              - Option 1: Use one of the supported models from the dropdown.
              - Option 2: Enter the ID of a custom model.

              Note: For custom models we currently only support models finetuned using using
              the Scale-hosted LLM-Engine API.

          prompt: Prompt for which to generate the completion.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing a more
              specific prompt here before trying more expensive techniques such as swapping in
              other models or finetuning the underlying LLM.

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          images: List of image urls to be used for image based completions. Leave empty for text
              based completions.

          model_parameters: Configuration parameters for the completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ],
        prompt: str,
        stream: bool,
        account_id: str | NotGiven = NOT_GIVEN,
        images: Iterable[completion_create_params.Image] | NotGiven = NOT_GIVEN,
        model_parameters: completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionsResponse | Stream[CompletionResponse]:
        """
        ### Description

        Given a user's input, runs LLM inference to produce the model's response.

        ### Details

        LLM [completions](https://scale-egp.readme.io/docs/completions-1) have many use
        cases, such as content summarization, question-answering, and text generation.

        The `model` parameter determines which LLM will be used to generate the
        completion. Keep in mind that different models have varying sizes, costs, and
        may perform differently across different tasks.

        The user input, commonly referred to as the "prompt", is a required field in the
        request body. The quality of the model's response can vary greatly depending on
        the input prompt. Good prompt engineering can significantly enhance the response
        quality. If you encounter suboptimal results, consider writing more specific
        instructions or providing examples to the LLM before trying more expensive
        techniques such as swapping in other models or finetuning.

        By default, the endpoint will return the entire response as one whole object. If
        you would prefer to stream the completion in real-time, you can achieve this by
        setting the `stream` flag to `true`.

        Args:
          model: The ID of the model to use for completions.

              Users have two options:

              - Option 1: Use one of the supported models from the dropdown.
              - Option 2: Enter the ID of a custom model.

              Note: For custom models we currently only support models finetuned using using
              the Scale-hosted LLM-Engine API.

          prompt: Prompt for which to generate the completion.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing a more
              specific prompt here before trying more expensive techniques such as swapping in
              other models or finetuning the underlying LLM.

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          images: List of image urls to be used for image based completions. Leave empty for text
              based completions.

          model_parameters: Configuration parameters for the completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["model", "prompt"], ["model", "prompt", "stream"])
    def create(
        self,
        *,
        model: Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ],
        prompt: str,
        account_id: str | NotGiven = NOT_GIVEN,
        images: Iterable[completion_create_params.Image] | NotGiven = NOT_GIVEN,
        model_parameters: completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        stream: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionsResponse | Stream[CompletionResponse]:
        return self._post(
            "/v4/completions",
            body=maybe_transform(
                {
                    "model": model,
                    "prompt": prompt,
                    "account_id": account_id,
                    "images": images,
                    "model_parameters": model_parameters,
                    "stream": stream,
                },
                completion_create_params.CompletionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompletionsResponse,
            stream=stream or False,
            stream_cls=Stream[CompletionResponse],
        )


class AsyncCompletionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncCompletionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        model: Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ],
        prompt: str,
        account_id: str | NotGiven = NOT_GIVEN,
        images: Iterable[completion_create_params.Image] | NotGiven = NOT_GIVEN,
        model_parameters: completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        stream: Literal[False] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionsResponse:
        """
        ### Description

        Given a user's input, runs LLM inference to produce the model's response.

        ### Details

        LLM [completions](https://scale-egp.readme.io/docs/completions-1) have many use
        cases, such as content summarization, question-answering, and text generation.

        The `model` parameter determines which LLM will be used to generate the
        completion. Keep in mind that different models have varying sizes, costs, and
        may perform differently across different tasks.

        The user input, commonly referred to as the "prompt", is a required field in the
        request body. The quality of the model's response can vary greatly depending on
        the input prompt. Good prompt engineering can significantly enhance the response
        quality. If you encounter suboptimal results, consider writing more specific
        instructions or providing examples to the LLM before trying more expensive
        techniques such as swapping in other models or finetuning.

        By default, the endpoint will return the entire response as one whole object. If
        you would prefer to stream the completion in real-time, you can achieve this by
        setting the `stream` flag to `true`.

        Args:
          model: The ID of the model to use for completions.

              Users have two options:

              - Option 1: Use one of the supported models from the dropdown.
              - Option 2: Enter the ID of a custom model.

              Note: For custom models we currently only support models finetuned using using
              the Scale-hosted LLM-Engine API.

          prompt: Prompt for which to generate the completion.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing a more
              specific prompt here before trying more expensive techniques such as swapping in
              other models or finetuning the underlying LLM.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          images: List of image urls to be used for image based completions. Leave empty for text
              based completions.

          model_parameters: Configuration parameters for the completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ],
        prompt: str,
        stream: Literal[True],
        account_id: str | NotGiven = NOT_GIVEN,
        images: Iterable[completion_create_params.Image] | NotGiven = NOT_GIVEN,
        model_parameters: completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncStream[CompletionResponse]:
        """
        ### Description

        Given a user's input, runs LLM inference to produce the model's response.

        ### Details

        LLM [completions](https://scale-egp.readme.io/docs/completions-1) have many use
        cases, such as content summarization, question-answering, and text generation.

        The `model` parameter determines which LLM will be used to generate the
        completion. Keep in mind that different models have varying sizes, costs, and
        may perform differently across different tasks.

        The user input, commonly referred to as the "prompt", is a required field in the
        request body. The quality of the model's response can vary greatly depending on
        the input prompt. Good prompt engineering can significantly enhance the response
        quality. If you encounter suboptimal results, consider writing more specific
        instructions or providing examples to the LLM before trying more expensive
        techniques such as swapping in other models or finetuning.

        By default, the endpoint will return the entire response as one whole object. If
        you would prefer to stream the completion in real-time, you can achieve this by
        setting the `stream` flag to `true`.

        Args:
          model: The ID of the model to use for completions.

              Users have two options:

              - Option 1: Use one of the supported models from the dropdown.
              - Option 2: Enter the ID of a custom model.

              Note: For custom models we currently only support models finetuned using using
              the Scale-hosted LLM-Engine API.

          prompt: Prompt for which to generate the completion.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing a more
              specific prompt here before trying more expensive techniques such as swapping in
              other models or finetuning the underlying LLM.

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          images: List of image urls to be used for image based completions. Leave empty for text
              based completions.

          model_parameters: Configuration parameters for the completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ],
        prompt: str,
        stream: bool,
        account_id: str | NotGiven = NOT_GIVEN,
        images: Iterable[completion_create_params.Image] | NotGiven = NOT_GIVEN,
        model_parameters: completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionsResponse | AsyncStream[CompletionResponse]:
        """
        ### Description

        Given a user's input, runs LLM inference to produce the model's response.

        ### Details

        LLM [completions](https://scale-egp.readme.io/docs/completions-1) have many use
        cases, such as content summarization, question-answering, and text generation.

        The `model` parameter determines which LLM will be used to generate the
        completion. Keep in mind that different models have varying sizes, costs, and
        may perform differently across different tasks.

        The user input, commonly referred to as the "prompt", is a required field in the
        request body. The quality of the model's response can vary greatly depending on
        the input prompt. Good prompt engineering can significantly enhance the response
        quality. If you encounter suboptimal results, consider writing more specific
        instructions or providing examples to the LLM before trying more expensive
        techniques such as swapping in other models or finetuning.

        By default, the endpoint will return the entire response as one whole object. If
        you would prefer to stream the completion in real-time, you can achieve this by
        setting the `stream` flag to `true`.

        Args:
          model: The ID of the model to use for completions.

              Users have two options:

              - Option 1: Use one of the supported models from the dropdown.
              - Option 2: Enter the ID of a custom model.

              Note: For custom models we currently only support models finetuned using using
              the Scale-hosted LLM-Engine API.

          prompt: Prompt for which to generate the completion.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing a more
              specific prompt here before trying more expensive techniques such as swapping in
              other models or finetuning the underlying LLM.

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          images: List of image urls to be used for image based completions. Leave empty for text
              based completions.

          model_parameters: Configuration parameters for the completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["model", "prompt"], ["model", "prompt", "stream"])
    async def create(
        self,
        *,
        model: Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ],
        prompt: str,
        account_id: str | NotGiven = NOT_GIVEN,
        images: Iterable[completion_create_params.Image] | NotGiven = NOT_GIVEN,
        model_parameters: completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        stream: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionsResponse | AsyncStream[CompletionResponse]:
        return await self._post(
            "/v4/completions",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "prompt": prompt,
                    "account_id": account_id,
                    "images": images,
                    "model_parameters": model_parameters,
                    "stream": stream,
                },
                completion_create_params.CompletionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompletionsResponse,
            stream=stream or False,
            stream_cls=AsyncStream[CompletionResponse],
        )


class CompletionsResourceWithRawResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_raw_response_wrapper(
            completions.create,
        )


class AsyncCompletionsResourceWithRawResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_raw_response_wrapper(
            completions.create,
        )


class CompletionsResourceWithStreamingResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_streamed_response_wrapper(
            completions.create,
        )


class AsyncCompletionsResourceWithStreamingResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_streamed_response_wrapper(
            completions.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TokenChunkingStrategyConfigParam"]


class TokenChunkingStrategyConfigParam(TypedDict, total=False):
    strategy: Required[Literal["token"]]

    chunk_overlap: int
    """Number of tokens to overlap between chunks.

    If not specified, an overlap of 0 will be used. Not this if only followed
    approximately.
    """

    max_chunk_size: int
    """Maximum number of tokens in each chunk.

    If not specified, a maximum chunk size of 600 will be used.
    """

    separator: str
    """Character designating breaks in input data.

    Text data will first be split into sections by this separator, then each section
    will be split into chunks of size `chunk_size`.
    """

    target_chunk_size: int
    """Target number of tokens in each chunk.

    If not specified, a target chunk size of 200 will be used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["S3DataSourceAuthConfigParam"]


class S3DataSourceAuthConfigParam(TypedDict, total=False):
    source: Required[Literal["S3"]]

    encrypted: bool

    external_id: str
    """External ID defined by the customer for the IAM role"""

    s3_role: str
    """Name of the role that a client will be initialized via AssumeRole of AWS sts"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .._models import BaseModel

__all__ = ["AutoGeneratedDraftTestCaseMissingChunkInfo"]


class AutoGeneratedDraftTestCaseMissingChunkInfo(BaseModel):
    artifact_content_modification_identifier: str
    """Artifact Content Modification Identifier of the missing chunk"""

    artifact_id: str
    """Artifact ID of the missing chunk"""

    artifact_name: str
    """Artifact Name of the missing chunk"""

    chunk_text: str
    """Text of chunk_id that is missing"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .fine_tuning_jobs import (
    FineTuningJobsResource,
    AsyncFineTuningJobsResource,
    FineTuningJobsResourceWithRawResponse,
    AsyncFineTuningJobsResourceWithRawResponse,
    FineTuningJobsResourceWithStreamingResponse,
    AsyncFineTuningJobsResourceWithStreamingResponse,
)

__all__ = [
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "FineTuningJobsResource",
    "AsyncFineTuningJobsResource",
    "FineTuningJobsResourceWithRawResponse",
    "AsyncFineTuningJobsResourceWithRawResponse",
    "FineTuningJobsResourceWithStreamingResponse",
    "AsyncFineTuningJobsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TaskUpdateParams"]


class TaskUpdateParams(TypedDict, total=False):
    evaluation_id: Required[str]

    assigned_to: str
    """The ID of the user that the task is assigned to."""



from __future__ import annotations

import pytest

from scale_gp import SGPClient
from scale_gp.lib.application_builder import ApplicationBuilder


class TestApplicationBuilderLibrary:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_create_completion_application(self, client: SGPClient) -> None:
        ApplicationBuilder(client).create_completion_application(
            account_id="account_id",
            application_name="test_application_name",
            completion_model_id="completion_model_id",
        )

    @parametrize
    def test_create_rag_application(self, client: SGPClient) -> None:
        ApplicationBuilder(client).create_rag_application(
            account_id="account_id",
            application_name="test_application_name",
            completion_model_id="completion_model_id",
            knowledge_base_id="knowledge_base_id",
            knowledge_base_top_k=1,
            prompt_engineering_template="prompt_engineering_template",
            reranker_model_id="reranker_model_id",
            reranker_top_k=1,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .test_case import TestCase as TestCase
from .flexible_chunk import FlexibleChunk as FlexibleChunk
from .flexible_message import FlexibleMessage as FlexibleMessage
from .flexible_chunk_param import FlexibleChunkParam as FlexibleChunkParam
from .paginated_test_cases import PaginatedTestCases as PaginatedTestCases
from .test_case_list_params import TestCaseListParams as TestCaseListParams
from .flexible_message_param import FlexibleMessageParam as FlexibleMessageParam
from .test_case_batch_params import TestCaseBatchParams as TestCaseBatchParams
from .test_case_create_params import TestCaseCreateParams as TestCaseCreateParams
from .test_case_update_params import TestCaseUpdateParams as TestCaseUpdateParams
from .test_case_batch_response import TestCaseBatchResponse as TestCaseBatchResponse
from .flexible_test_case_schema import FlexibleTestCaseSchema as FlexibleTestCaseSchema
from .artifact_schema_generation import ArtifactSchemaGeneration as ArtifactSchemaGeneration
from .evaluation_dataset_version import EvaluationDatasetVersion as EvaluationDatasetVersion
from .generation_test_case_schema import GenerationTestCaseSchema as GenerationTestCaseSchema
from .generation_job_create_params import GenerationJobCreateParams as GenerationJobCreateParams
from .auto_generated_draft_test_case import AutoGeneratedDraftTestCase as AutoGeneratedDraftTestCase
from .flexible_test_case_schema_param import FlexibleTestCaseSchemaParam as FlexibleTestCaseSchemaParam
from .artifact_schema_generation_param import ArtifactSchemaGenerationParam as ArtifactSchemaGenerationParam
from .evaluation_dataset_generation_job import EvaluationDatasetGenerationJob as EvaluationDatasetGenerationJob
from .generation_test_case_schema_param import GenerationTestCaseSchemaParam as GenerationTestCaseSchemaParam
from .auto_generated_draft_test_cases_list import AutoGeneratedDraftTestCasesList as AutoGeneratedDraftTestCasesList
from .paginated_evaluation_dataset_versions import (
    PaginatedEvaluationDatasetVersions as PaginatedEvaluationDatasetVersions,
)
from .evaluation_dataset_version_list_params import (
    EvaluationDatasetVersionListParams as EvaluationDatasetVersionListParams,
)
from .evaluation_dataset_generation_jobs_list import (
    EvaluationDatasetGenerationJobsList as EvaluationDatasetGenerationJobsList,
)
from .evaluation_dataset_version_create_params import (
    EvaluationDatasetVersionCreateParams as EvaluationDatasetVersionCreateParams,
)
from .autogenerated_draft_test_case_list_params import (
    AutogeneratedDraftTestCaseListParams as AutogeneratedDraftTestCaseListParams,
)
from .evaluation_dataset_generation_job_response import (
    EvaluationDatasetGenerationJobResponse as EvaluationDatasetGenerationJobResponse,
)
from .autogenerated_draft_test_case_create_params import (
    AutogeneratedDraftTestCaseCreateParams as AutogeneratedDraftTestCaseCreateParams,
)
from .autogenerated_draft_test_case_list_response import (
    AutogeneratedDraftTestCaseListResponse as AutogeneratedDraftTestCaseListResponse,
)
from .autogenerated_draft_test_case_update_params import (
    AutogeneratedDraftTestCaseUpdateParams as AutogeneratedDraftTestCaseUpdateParams,
)
from .autogenerated_draft_test_case_approve_params import (
    AutogeneratedDraftTestCaseApproveParams as AutogeneratedDraftTestCaseApproveParams,
)
from .approve_auto_generated_draft_test_case_response import (
    ApproveAutoGeneratedDraftTestCaseResponse as ApproveAutoGeneratedDraftTestCaseResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .test_case_result import TestCaseResult

__all__ = ["TestCaseResultBatchResponse"]

TestCaseResultBatchResponse: TypeAlias = List[TestCaseResult]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Mapping, Iterable, cast
from typing_extensions import Literal

import httpx

from ...types import (
    application_process_params,
    application_validate_params,
    application_upload_files_params,
)
from .metrics import (
    MetricsResource,
    AsyncMetricsResource,
    MetricsResourceWithRawResponse,
    AsyncMetricsResourceWithRawResponse,
    MetricsResourceWithStreamingResponse,
    AsyncMetricsResourceWithStreamingResponse,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from ..._utils import (
    extract_files,
    maybe_transform,
    deepcopy_minimal,
    async_maybe_transform,
)
from ..._compat import cached_property
from .dashboards import (
    DashboardsResource,
    AsyncDashboardsResource,
    DashboardsResourceWithRawResponse,
    AsyncDashboardsResourceWithRawResponse,
    DashboardsResourceWithStreamingResponse,
    AsyncDashboardsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .chat_threads import (
    ChatThreadsResource,
    AsyncChatThreadsResource,
    ChatThreadsResourceWithRawResponse,
    AsyncChatThreadsResourceWithRawResponse,
    ChatThreadsResourceWithStreamingResponse,
    AsyncChatThreadsResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from .metrics.metrics import MetricsResource, AsyncMetricsResource
from .chat_threads.chat_threads import ChatThreadsResource, AsyncChatThreadsResource
from ...types.application_edge_param import ApplicationEdgeParam
from ...types.application_node_param import ApplicationNodeParam
from ...types.application_upload_files_response import ApplicationUploadFilesResponse

__all__ = ["ApplicationsResource", "AsyncApplicationsResource"]


class ApplicationsResource(SyncAPIResource):
    @cached_property
    def chat_threads(self) -> ChatThreadsResource:
        return ChatThreadsResource(self._client)

    @cached_property
    def dashboards(self) -> DashboardsResource:
        return DashboardsResource(self._client)

    @cached_property
    def metrics(self) -> MetricsResource:
        return MetricsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ApplicationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ApplicationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ApplicationsResourceWithStreamingResponse(self)

    def process(
        self,
        *,
        edges: Iterable[ApplicationEdgeParam],
        inputs: Dict[str, object],
        nodes: Iterable[ApplicationNodeParam],
        version: Literal["V0"],
        history: Iterable[application_process_params.History] | NotGiven = NOT_GIVEN,
        overrides: Dict[str, application_process_params.Overrides] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Process Application

        Args:
          edges: List of edges in the application graph

          inputs: Input data for the application. You must provide inputs for each input node

          nodes: List of nodes in the application graph

          version: Version of the application schema

          history: History of the application

          overrides: Optional overrides for the application

          stream: Control to have streaming of the endpoint. If the last node before the output is
              a completion node, you can set this to true to get the output as soon as the
              completion node has a token

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/applications/process",
            body=maybe_transform(
                {
                    "edges": edges,
                    "inputs": inputs,
                    "nodes": nodes,
                    "version": version,
                    "history": history,
                    "overrides": overrides,
                    "stream": stream,
                },
                application_process_params.ApplicationProcessParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def upload_files(
        self,
        *,
        files: List[FileTypes],
        account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationUploadFilesResponse:
        """
        Upload Application Files

        Args:
          files: Upload files to be used in an application.

          account_id: Account which the file will be tied to. Use this account id query param if you
              are using the API or the SDK.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal({"files": files})
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/v4/applications/upload-files",
            body=maybe_transform(body, application_upload_files_params.ApplicationUploadFilesParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"account_id": account_id}, application_upload_files_params.ApplicationUploadFilesParams
                ),
            ),
            cast_to=ApplicationUploadFilesResponse,
        )

    def validate(
        self,
        *,
        edges: Iterable[ApplicationEdgeParam],
        nodes: Iterable[ApplicationNodeParam],
        version: Literal["V0"],
        overrides: Dict[str, application_validate_params.Overrides] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Validate Application

        Args:
          edges: List of edges in the application graph

          nodes: List of nodes in the application graph

          version: Version of the application schema

          overrides: Optional overrides for the application

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/applications/validate",
            body=maybe_transform(
                {
                    "edges": edges,
                    "nodes": nodes,
                    "version": version,
                    "overrides": overrides,
                },
                application_validate_params.ApplicationValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncApplicationsResource(AsyncAPIResource):
    @cached_property
    def chat_threads(self) -> AsyncChatThreadsResource:
        return AsyncChatThreadsResource(self._client)

    @cached_property
    def dashboards(self) -> AsyncDashboardsResource:
        return AsyncDashboardsResource(self._client)

    @cached_property
    def metrics(self) -> AsyncMetricsResource:
        return AsyncMetricsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncApplicationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncApplicationsResourceWithStreamingResponse(self)

    async def process(
        self,
        *,
        edges: Iterable[ApplicationEdgeParam],
        inputs: Dict[str, object],
        nodes: Iterable[ApplicationNodeParam],
        version: Literal["V0"],
        history: Iterable[application_process_params.History] | NotGiven = NOT_GIVEN,
        overrides: Dict[str, application_process_params.Overrides] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Process Application

        Args:
          edges: List of edges in the application graph

          inputs: Input data for the application. You must provide inputs for each input node

          nodes: List of nodes in the application graph

          version: Version of the application schema

          history: History of the application

          overrides: Optional overrides for the application

          stream: Control to have streaming of the endpoint. If the last node before the output is
              a completion node, you can set this to true to get the output as soon as the
              completion node has a token

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/applications/process",
            body=await async_maybe_transform(
                {
                    "edges": edges,
                    "inputs": inputs,
                    "nodes": nodes,
                    "version": version,
                    "history": history,
                    "overrides": overrides,
                    "stream": stream,
                },
                application_process_params.ApplicationProcessParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def upload_files(
        self,
        *,
        files: List[FileTypes],
        account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationUploadFilesResponse:
        """
        Upload Application Files

        Args:
          files: Upload files to be used in an application.

          account_id: Account which the file will be tied to. Use this account id query param if you
              are using the API or the SDK.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal({"files": files})
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/v4/applications/upload-files",
            body=await async_maybe_transform(body, application_upload_files_params.ApplicationUploadFilesParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"account_id": account_id}, application_upload_files_params.ApplicationUploadFilesParams
                ),
            ),
            cast_to=ApplicationUploadFilesResponse,
        )

    async def validate(
        self,
        *,
        edges: Iterable[ApplicationEdgeParam],
        nodes: Iterable[ApplicationNodeParam],
        version: Literal["V0"],
        overrides: Dict[str, application_validate_params.Overrides] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Validate Application

        Args:
          edges: List of edges in the application graph

          nodes: List of nodes in the application graph

          version: Version of the application schema

          overrides: Optional overrides for the application

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/applications/validate",
            body=await async_maybe_transform(
                {
                    "edges": edges,
                    "nodes": nodes,
                    "version": version,
                    "overrides": overrides,
                },
                application_validate_params.ApplicationValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ApplicationsResourceWithRawResponse:
    def __init__(self, applications: ApplicationsResource) -> None:
        self._applications = applications

        self.process = to_raw_response_wrapper(
            applications.process,
        )
        self.upload_files = to_raw_response_wrapper(
            applications.upload_files,
        )
        self.validate = to_raw_response_wrapper(
            applications.validate,
        )

    @cached_property
    def chat_threads(self) -> ChatThreadsResourceWithRawResponse:
        return ChatThreadsResourceWithRawResponse(self._applications.chat_threads)

    @cached_property
    def dashboards(self) -> DashboardsResourceWithRawResponse:
        return DashboardsResourceWithRawResponse(self._applications.dashboards)

    @cached_property
    def metrics(self) -> MetricsResourceWithRawResponse:
        return MetricsResourceWithRawResponse(self._applications.metrics)


class AsyncApplicationsResourceWithRawResponse:
    def __init__(self, applications: AsyncApplicationsResource) -> None:
        self._applications = applications

        self.process = async_to_raw_response_wrapper(
            applications.process,
        )
        self.upload_files = async_to_raw_response_wrapper(
            applications.upload_files,
        )
        self.validate = async_to_raw_response_wrapper(
            applications.validate,
        )

    @cached_property
    def chat_threads(self) -> AsyncChatThreadsResourceWithRawResponse:
        return AsyncChatThreadsResourceWithRawResponse(self._applications.chat_threads)

    @cached_property
    def dashboards(self) -> AsyncDashboardsResourceWithRawResponse:
        return AsyncDashboardsResourceWithRawResponse(self._applications.dashboards)

    @cached_property
    def metrics(self) -> AsyncMetricsResourceWithRawResponse:
        return AsyncMetricsResourceWithRawResponse(self._applications.metrics)


class ApplicationsResourceWithStreamingResponse:
    def __init__(self, applications: ApplicationsResource) -> None:
        self._applications = applications

        self.process = to_streamed_response_wrapper(
            applications.process,
        )
        self.upload_files = to_streamed_response_wrapper(
            applications.upload_files,
        )
        self.validate = to_streamed_response_wrapper(
            applications.validate,
        )

    @cached_property
    def chat_threads(self) -> ChatThreadsResourceWithStreamingResponse:
        return ChatThreadsResourceWithStreamingResponse(self._applications.chat_threads)

    @cached_property
    def dashboards(self) -> DashboardsResourceWithStreamingResponse:
        return DashboardsResourceWithStreamingResponse(self._applications.dashboards)

    @cached_property
    def metrics(self) -> MetricsResourceWithStreamingResponse:
        return MetricsResourceWithStreamingResponse(self._applications.metrics)


class AsyncApplicationsResourceWithStreamingResponse:
    def __init__(self, applications: AsyncApplicationsResource) -> None:
        self._applications = applications

        self.process = async_to_streamed_response_wrapper(
            applications.process,
        )
        self.upload_files = async_to_streamed_response_wrapper(
            applications.upload_files,
        )
        self.validate = async_to_streamed_response_wrapper(
            applications.validate,
        )

    @cached_property
    def chat_threads(self) -> AsyncChatThreadsResourceWithStreamingResponse:
        return AsyncChatThreadsResourceWithStreamingResponse(self._applications.chat_threads)

    @cached_property
    def dashboards(self) -> AsyncDashboardsResourceWithStreamingResponse:
        return AsyncDashboardsResourceWithStreamingResponse(self._applications.dashboards)

    @cached_property
    def metrics(self) -> AsyncMetricsResourceWithStreamingResponse:
        return AsyncMetricsResourceWithStreamingResponse(self._applications.metrics)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import Theme
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestThemes:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        theme = client.themes.create(
            account_id="account_id",
            logo_blob="logo_blob",
            theme_vars={},
            title="title",
        )
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        theme = client.themes.create(
            account_id="account_id",
            logo_blob="logo_blob",
            theme_vars={
                "accent_primary": "accentPrimary",
                "accent_secondary": "accentSecondary",
                "background": "background",
                "foreground": "foreground",
            },
            title="title",
        )
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.themes.with_raw_response.create(
            account_id="account_id",
            logo_blob="logo_blob",
            theme_vars={},
            title="title",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        theme = response.parse()
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.themes.with_streaming_response.create(
            account_id="account_id",
            logo_blob="logo_blob",
            theme_vars={},
            title="title",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            theme = response.parse()
            assert_matches_type(Theme, theme, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        theme = client.themes.retrieve(
            "theme_id",
        )
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.themes.with_raw_response.retrieve(
            "theme_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        theme = response.parse()
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.themes.with_streaming_response.retrieve(
            "theme_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            theme = response.parse()
            assert_matches_type(Theme, theme, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `theme_id` but received ''"):
            client.themes.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        theme = client.themes.list()
        assert_matches_type(SyncPageResponse[Theme], theme, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        theme = client.themes.list(
            account_id="account_id",
            include_archived=True,
            limit=1,
            page=1,
        )
        assert_matches_type(SyncPageResponse[Theme], theme, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.themes.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        theme = response.parse()
        assert_matches_type(SyncPageResponse[Theme], theme, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.themes.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            theme = response.parse()
            assert_matches_type(SyncPageResponse[Theme], theme, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncThemes:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        theme = await async_client.themes.create(
            account_id="account_id",
            logo_blob="logo_blob",
            theme_vars={},
            title="title",
        )
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        theme = await async_client.themes.create(
            account_id="account_id",
            logo_blob="logo_blob",
            theme_vars={
                "accent_primary": "accentPrimary",
                "accent_secondary": "accentSecondary",
                "background": "background",
                "foreground": "foreground",
            },
            title="title",
        )
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.themes.with_raw_response.create(
            account_id="account_id",
            logo_blob="logo_blob",
            theme_vars={},
            title="title",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        theme = await response.parse()
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.themes.with_streaming_response.create(
            account_id="account_id",
            logo_blob="logo_blob",
            theme_vars={},
            title="title",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            theme = await response.parse()
            assert_matches_type(Theme, theme, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        theme = await async_client.themes.retrieve(
            "theme_id",
        )
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.themes.with_raw_response.retrieve(
            "theme_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        theme = await response.parse()
        assert_matches_type(Theme, theme, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.themes.with_streaming_response.retrieve(
            "theme_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            theme = await response.parse()
            assert_matches_type(Theme, theme, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `theme_id` but received ''"):
            await async_client.themes.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        theme = await async_client.themes.list()
        assert_matches_type(AsyncPageResponse[Theme], theme, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        theme = await async_client.themes.list(
            account_id="account_id",
            include_archived=True,
            limit=1,
            page=1,
        )
        assert_matches_type(AsyncPageResponse[Theme], theme, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.themes.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        theme = await response.parse()
        assert_matches_type(AsyncPageResponse[Theme], theme, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.themes.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            theme = await response.parse()
            assert_matches_type(AsyncPageResponse[Theme], theme, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    knowledge_base_data_source_list_params,
    knowledge_base_data_source_create_params,
    knowledge_base_data_source_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.knowledge_base_data_source import KnowledgeBaseDataSource
from ..types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["KnowledgeBaseDataSourcesResource", "AsyncKnowledgeBaseDataSourcesResource"]


class KnowledgeBaseDataSourcesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> KnowledgeBaseDataSourcesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return KnowledgeBaseDataSourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> KnowledgeBaseDataSourcesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return KnowledgeBaseDataSourcesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        data_source_config: knowledge_base_data_source_create_params.DataSourceConfig,
        name: str,
        data_source_auth_config: knowledge_base_data_source_create_params.DataSourceAuthConfig | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseDataSource:
        """
        ### Description

        Creates a knowledge base data source

        ### Details

        This API can be used to create a knowledge base data source. To use this API,
        review the request schema and pass in all fields that are required to create a
        knowledge base data source.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/knowledge-base-data-sources",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "data_source_config": data_source_config,
                    "name": name,
                    "data_source_auth_config": data_source_auth_config,
                    "description": description,
                },
                knowledge_base_data_source_create_params.KnowledgeBaseDataSourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseDataSource,
        )

    def retrieve(
        self,
        knowledge_base_data_source_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseDataSource:
        """
        ### Description

        Gets the details of a knowledge base data source

        ### Details

        This API can be used to get information about a single knowledge base data
        source by ID. To use this API, pass in the `id` that was returned from your
        Create Knowledge Base Data Source API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return self._get(
            f"/v4/knowledge-base-data-sources/{knowledge_base_data_source_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseDataSource,
        )

    def update(
        self,
        knowledge_base_data_source_id: str,
        *,
        data_source_auth_config: knowledge_base_data_source_update_params.DataSourceAuthConfig | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseDataSource:
        """
        ### Description

        Updates a knowledge base data source

        ### Details

        This API can be used to update the knowledge base data source that matches the
        ID that was passed in as a path parameter. To use this API, pass in the `id`
        that was returned from your Create Knowledge Base Data Source API call as a path
        parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return self._patch(
            f"/v4/knowledge-base-data-sources/{knowledge_base_data_source_id}",
            body=maybe_transform(
                {
                    "data_source_auth_config": data_source_auth_config,
                    "description": description,
                    "name": name,
                },
                knowledge_base_data_source_update_params.KnowledgeBaseDataSourceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseDataSource,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[KnowledgeBaseDataSource]:
        """
        ### Description

        Lists all knowledge base data sources accessible to the user.

        ### Details

        This API can be used to list knowledge base data sources. If a user has access
        to multiple accounts, all knowledge base data sources from all accounts the user
        is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/knowledge-base-data-sources",
            page=SyncPageResponse[KnowledgeBaseDataSource],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    knowledge_base_data_source_list_params.KnowledgeBaseDataSourceListParams,
                ),
            ),
            model=KnowledgeBaseDataSource,
        )

    def delete(
        self,
        knowledge_base_data_source_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a knowledge base data source

        ### Details

        This API can be used to delete a knowledge base data source by ID. To use this
        API, pass in the `id` that was returned from your Create Knowledge Base Data
        Source API call as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return self._delete(
            f"/v4/knowledge-base-data-sources/{knowledge_base_data_source_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    def verify(
        self,
        knowledge_base_data_source_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Test Knowledge Base Data Source credentials

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return self._post(
            f"/v4/knowledge-base-data-sources/{knowledge_base_data_source_id}/verify",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncKnowledgeBaseDataSourcesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncKnowledgeBaseDataSourcesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncKnowledgeBaseDataSourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncKnowledgeBaseDataSourcesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncKnowledgeBaseDataSourcesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        data_source_config: knowledge_base_data_source_create_params.DataSourceConfig,
        name: str,
        data_source_auth_config: knowledge_base_data_source_create_params.DataSourceAuthConfig | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseDataSource:
        """
        ### Description

        Creates a knowledge base data source

        ### Details

        This API can be used to create a knowledge base data source. To use this API,
        review the request schema and pass in all fields that are required to create a
        knowledge base data source.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/knowledge-base-data-sources",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "data_source_config": data_source_config,
                    "name": name,
                    "data_source_auth_config": data_source_auth_config,
                    "description": description,
                },
                knowledge_base_data_source_create_params.KnowledgeBaseDataSourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseDataSource,
        )

    async def retrieve(
        self,
        knowledge_base_data_source_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseDataSource:
        """
        ### Description

        Gets the details of a knowledge base data source

        ### Details

        This API can be used to get information about a single knowledge base data
        source by ID. To use this API, pass in the `id` that was returned from your
        Create Knowledge Base Data Source API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return await self._get(
            f"/v4/knowledge-base-data-sources/{knowledge_base_data_source_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseDataSource,
        )

    async def update(
        self,
        knowledge_base_data_source_id: str,
        *,
        data_source_auth_config: knowledge_base_data_source_update_params.DataSourceAuthConfig | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseDataSource:
        """
        ### Description

        Updates a knowledge base data source

        ### Details

        This API can be used to update the knowledge base data source that matches the
        ID that was passed in as a path parameter. To use this API, pass in the `id`
        that was returned from your Create Knowledge Base Data Source API call as a path
        parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return await self._patch(
            f"/v4/knowledge-base-data-sources/{knowledge_base_data_source_id}",
            body=await async_maybe_transform(
                {
                    "data_source_auth_config": data_source_auth_config,
                    "description": description,
                    "name": name,
                },
                knowledge_base_data_source_update_params.KnowledgeBaseDataSourceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseDataSource,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[KnowledgeBaseDataSource, AsyncPageResponse[KnowledgeBaseDataSource]]:
        """
        ### Description

        Lists all knowledge base data sources accessible to the user.

        ### Details

        This API can be used to list knowledge base data sources. If a user has access
        to multiple accounts, all knowledge base data sources from all accounts the user
        is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/knowledge-base-data-sources",
            page=AsyncPageResponse[KnowledgeBaseDataSource],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    knowledge_base_data_source_list_params.KnowledgeBaseDataSourceListParams,
                ),
            ),
            model=KnowledgeBaseDataSource,
        )

    async def delete(
        self,
        knowledge_base_data_source_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a knowledge base data source

        ### Details

        This API can be used to delete a knowledge base data source by ID. To use this
        API, pass in the `id` that was returned from your Create Knowledge Base Data
        Source API call as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return await self._delete(
            f"/v4/knowledge-base-data-sources/{knowledge_base_data_source_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    async def verify(
        self,
        knowledge_base_data_source_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Test Knowledge Base Data Source credentials

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return await self._post(
            f"/v4/knowledge-base-data-sources/{knowledge_base_data_source_id}/verify",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class KnowledgeBaseDataSourcesResourceWithRawResponse:
    def __init__(self, knowledge_base_data_sources: KnowledgeBaseDataSourcesResource) -> None:
        self._knowledge_base_data_sources = knowledge_base_data_sources

        self.create = to_raw_response_wrapper(
            knowledge_base_data_sources.create,
        )
        self.retrieve = to_raw_response_wrapper(
            knowledge_base_data_sources.retrieve,
        )
        self.update = to_raw_response_wrapper(
            knowledge_base_data_sources.update,
        )
        self.list = to_raw_response_wrapper(
            knowledge_base_data_sources.list,
        )
        self.delete = to_raw_response_wrapper(
            knowledge_base_data_sources.delete,
        )
        self.verify = to_raw_response_wrapper(
            knowledge_base_data_sources.verify,
        )


class AsyncKnowledgeBaseDataSourcesResourceWithRawResponse:
    def __init__(self, knowledge_base_data_sources: AsyncKnowledgeBaseDataSourcesResource) -> None:
        self._knowledge_base_data_sources = knowledge_base_data_sources

        self.create = async_to_raw_response_wrapper(
            knowledge_base_data_sources.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            knowledge_base_data_sources.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            knowledge_base_data_sources.update,
        )
        self.list = async_to_raw_response_wrapper(
            knowledge_base_data_sources.list,
        )
        self.delete = async_to_raw_response_wrapper(
            knowledge_base_data_sources.delete,
        )
        self.verify = async_to_raw_response_wrapper(
            knowledge_base_data_sources.verify,
        )


class KnowledgeBaseDataSourcesResourceWithStreamingResponse:
    def __init__(self, knowledge_base_data_sources: KnowledgeBaseDataSourcesResource) -> None:
        self._knowledge_base_data_sources = knowledge_base_data_sources

        self.create = to_streamed_response_wrapper(
            knowledge_base_data_sources.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            knowledge_base_data_sources.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            knowledge_base_data_sources.update,
        )
        self.list = to_streamed_response_wrapper(
            knowledge_base_data_sources.list,
        )
        self.delete = to_streamed_response_wrapper(
            knowledge_base_data_sources.delete,
        )
        self.verify = to_streamed_response_wrapper(
            knowledge_base_data_sources.verify,
        )


class AsyncKnowledgeBaseDataSourcesResourceWithStreamingResponse:
    def __init__(self, knowledge_base_data_sources: AsyncKnowledgeBaseDataSourcesResource) -> None:
        self._knowledge_base_data_sources = knowledge_base_data_sources

        self.create = async_to_streamed_response_wrapper(
            knowledge_base_data_sources.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            knowledge_base_data_sources.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            knowledge_base_data_sources.update,
        )
        self.list = async_to_streamed_response_wrapper(
            knowledge_base_data_sources.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            knowledge_base_data_sources.delete,
        )
        self.verify = async_to_streamed_response_wrapper(
            knowledge_base_data_sources.verify,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat_threads import (
    ChatThreadsResource,
    AsyncChatThreadsResource,
    ChatThreadsResourceWithRawResponse,
    AsyncChatThreadsResourceWithRawResponse,
    ChatThreadsResourceWithStreamingResponse,
    AsyncChatThreadsResourceWithStreamingResponse,
)

__all__ = [
    "ChatThreadsResource",
    "AsyncChatThreadsResource",
    "ChatThreadsResourceWithRawResponse",
    "AsyncChatThreadsResourceWithRawResponse",
    "ChatThreadsResourceWithStreamingResponse",
    "AsyncChatThreadsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .test_case_result import TestCaseResult
from ..evaluation_datasets.test_case import TestCase
from ..shared.chunk_extra_info_schema import ChunkExtraInfoSchema
from ..shared.result_schema_generation import ResultSchemaGeneration
from ..shared.string_extra_info_schema import StringExtraInfoSchema
from ..evaluation_datasets.flexible_chunk import FlexibleChunk
from ..evaluation_datasets.flexible_message import FlexibleMessage

__all__ = [
    "TestCaseResultWithViews",
    "GenerationTestCaseResultResponseWithViews",
    "GenerationTestCaseResultResponseWithViewsAnnotationResult",
    "GenerationTestCaseResultResponseWithViewsAnnotationResultLlmAutoEvalMetadata",
    "GenerationTestCaseResultResponseWithViewsTask",
    "GenerationTestCaseResultResponseWithViewsTaskAssignedTo",
    "FlexibleTestCaseResultResponseWithViews",
    "FlexibleTestCaseResultResponseWithViewsTestCaseEvaluationData",
    "FlexibleTestCaseResultResponseWithViewsTestCaseEvaluationDataGenerationExtraInfo",
    "FlexibleTestCaseResultResponseWithViewsAnnotationResult",
    "FlexibleTestCaseResultResponseWithViewsAnnotationResultLlmAutoEvalMetadata",
    "FlexibleTestCaseResultResponseWithViewsTask",
    "FlexibleTestCaseResultResponseWithViewsTaskAssignedTo",
]


class GenerationTestCaseResultResponseWithViewsAnnotationResultLlmAutoEvalMetadata(BaseModel):
    annotation_result_id: str
    """The ID of the associated annotation result."""

    completion_tokens: int

    llm_reasoning: str
    """The reasoning the LLM gave for the annotation it provided."""

    prompt_tokens: int

    time_elapsed_s: int
    """The time elapsed to generate this annotation in seconds."""

    cost: Optional[int] = None
    """The cost of the annotation in cents."""


class GenerationTestCaseResultResponseWithViewsAnnotationResult(BaseModel):
    id: str
    """The unique identifier of the entity."""

    annotation_type: Literal["llm_auto", "human"]
    """An enumeration."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    question_id: str

    selected_choice: object
    """The selected choices(s) for the annotation result, in JSON form.

    For categorical questions, this is an object or list of objects (depending on if
    multiple selections are allowed). For free text questions, this is a string.
    """

    test_case_result_lineage_id: str

    llm_auto_eval_metadata: Optional[GenerationTestCaseResultResponseWithViewsAnnotationResultLlmAutoEvalMetadata] = (
        None
    )


class GenerationTestCaseResultResponseWithViewsTaskAssignedTo(BaseModel):
    id: str

    email: str

    first_name: Optional[str] = None

    last_name: Optional[str] = None


class GenerationTestCaseResultResponseWithViewsTask(BaseModel):
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    priority: int

    status: Literal["PENDING", "COMPLETED"]
    """An enumeration."""

    task_entity_id: str

    task_entity_parent_id: str

    task_type: Literal["EVALUATION_ANNOTATION"]
    """An enumeration."""

    assigned_to: Optional[GenerationTestCaseResultResponseWithViewsTaskAssignedTo] = None

    assignment_expires_at: Optional[datetime] = None
    """The date and time when the task assignment expires in ISO format."""

    task_entity: Optional[TestCaseResult] = None
    """The entity that the task is associated with."""


class GenerationTestCaseResultResponseWithViews(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset_id: str

    evaluation_dataset_version_num: str

    evaluation_id: str

    label_status: Literal["PENDING", "COMPLETED", "FAILED"]
    """An enumeration."""

    test_case_evaluation_data: ResultSchemaGeneration

    test_case_id: str

    annotated_by_user_id: Optional[str] = None
    """The user who annotated the task."""

    annotation_results: Optional[List[GenerationTestCaseResultResponseWithViewsAnnotationResult]] = None

    application_test_case_output_id: Optional[str] = None

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    audit_comment: Optional[str] = None

    audit_required: Optional[bool] = None

    audit_status: Optional[Literal["UNAUDITED", "FIXED", "APPROVED"]] = None
    """An enumeration."""

    completed_at: Optional[datetime] = None

    metrics: Optional[Dict[str, float]] = None

    result: Optional[object] = None

    task: Optional[GenerationTestCaseResultResponseWithViewsTask] = None

    test_case_evaluation_data_schema: Optional[Literal["GENERATION"]] = None

    test_case_version: Optional[TestCase] = None

    time_spent_labeling_s: Optional[int] = None
    """The time spent labeling in seconds."""


FlexibleTestCaseResultResponseWithViewsTestCaseEvaluationDataGenerationExtraInfo: TypeAlias = Annotated[
    Union[ChunkExtraInfoSchema, StringExtraInfoSchema], PropertyInfo(discriminator="schema_type")
]


class FlexibleTestCaseResultResponseWithViewsTestCaseEvaluationData(BaseModel):
    generation_output: Union[
        str,
        float,
        List[FlexibleChunk],
        List[FlexibleMessage],
        List[object],
        Dict[str, Union[str, float, List[FlexibleChunk], List[FlexibleMessage], List[object], object]],
        object,
    ]

    generation_extra_info: Optional[
        FlexibleTestCaseResultResponseWithViewsTestCaseEvaluationDataGenerationExtraInfo
    ] = None


class FlexibleTestCaseResultResponseWithViewsAnnotationResultLlmAutoEvalMetadata(BaseModel):
    annotation_result_id: str
    """The ID of the associated annotation result."""

    completion_tokens: int

    llm_reasoning: str
    """The reasoning the LLM gave for the annotation it provided."""

    prompt_tokens: int

    time_elapsed_s: int
    """The time elapsed to generate this annotation in seconds."""

    cost: Optional[int] = None
    """The cost of the annotation in cents."""


class FlexibleTestCaseResultResponseWithViewsAnnotationResult(BaseModel):
    id: str
    """The unique identifier of the entity."""

    annotation_type: Literal["llm_auto", "human"]
    """An enumeration."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    question_id: str

    selected_choice: object
    """The selected choices(s) for the annotation result, in JSON form.

    For categorical questions, this is an object or list of objects (depending on if
    multiple selections are allowed). For free text questions, this is a string.
    """

    test_case_result_lineage_id: str

    llm_auto_eval_metadata: Optional[FlexibleTestCaseResultResponseWithViewsAnnotationResultLlmAutoEvalMetadata] = None


class FlexibleTestCaseResultResponseWithViewsTaskAssignedTo(BaseModel):
    id: str

    email: str

    first_name: Optional[str] = None

    last_name: Optional[str] = None


class FlexibleTestCaseResultResponseWithViewsTask(BaseModel):
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    priority: int

    status: Literal["PENDING", "COMPLETED"]
    """An enumeration."""

    task_entity_id: str

    task_entity_parent_id: str

    task_type: Literal["EVALUATION_ANNOTATION"]
    """An enumeration."""

    assigned_to: Optional[FlexibleTestCaseResultResponseWithViewsTaskAssignedTo] = None

    assignment_expires_at: Optional[datetime] = None
    """The date and time when the task assignment expires in ISO format."""

    task_entity: Optional[TestCaseResult] = None
    """The entity that the task is associated with."""


class FlexibleTestCaseResultResponseWithViews(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset_id: str

    evaluation_dataset_version_num: str

    evaluation_id: str

    label_status: Literal["PENDING", "COMPLETED", "FAILED"]
    """An enumeration."""

    test_case_evaluation_data: FlexibleTestCaseResultResponseWithViewsTestCaseEvaluationData

    test_case_id: str

    annotated_by_user_id: Optional[str] = None
    """The user who annotated the task."""

    annotation_results: Optional[List[FlexibleTestCaseResultResponseWithViewsAnnotationResult]] = None

    application_test_case_output_id: Optional[str] = None

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    audit_comment: Optional[str] = None

    audit_required: Optional[bool] = None

    audit_status: Optional[Literal["UNAUDITED", "FIXED", "APPROVED"]] = None
    """An enumeration."""

    completed_at: Optional[datetime] = None

    metrics: Optional[Dict[str, float]] = None

    result: Optional[object] = None

    task: Optional[FlexibleTestCaseResultResponseWithViewsTask] = None

    test_case_evaluation_data_schema: Optional[Literal["FLEXIBLE"]] = None

    test_case_version: Optional[TestCase] = None

    time_spent_labeling_s: Optional[int] = None
    """The time spent labeling in seconds."""


TestCaseResultWithViews: TypeAlias = Annotated[
    Union[GenerationTestCaseResultResponseWithViews, FlexibleTestCaseResultResponseWithViews],
    PropertyInfo(discriminator="test_case_evaluation_data_schema"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.task import Task
from ..._base_client import make_request_options
from ...types.evaluations import task_update_params

__all__ = ["TasksResource", "AsyncTasksResource"]


class TasksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return TasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return TasksResourceWithStreamingResponse(self)

    def update(
        self,
        task_id: str,
        *,
        evaluation_id: str,
        assigned_to: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Task:
        """
        Update Evaluation Task

        Args:
          assigned_to: The ID of the user that the task is assigned to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._patch(
            f"/v4/evaluations/{evaluation_id}/tasks/{task_id}",
            body=maybe_transform({"assigned_to": assigned_to}, task_update_params.TaskUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Task,
        )


class AsyncTasksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncTasksResourceWithStreamingResponse(self)

    async def update(
        self,
        task_id: str,
        *,
        evaluation_id: str,
        assigned_to: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Task:
        """
        Update Evaluation Task

        Args:
          assigned_to: The ID of the user that the task is assigned to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._patch(
            f"/v4/evaluations/{evaluation_id}/tasks/{task_id}",
            body=await async_maybe_transform({"assigned_to": assigned_to}, task_update_params.TaskUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Task,
        )


class TasksResourceWithRawResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.update = to_raw_response_wrapper(
            tasks.update,
        )


class AsyncTasksResourceWithRawResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.update = async_to_raw_response_wrapper(
            tasks.update,
        )


class TasksResourceWithStreamingResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.update = to_streamed_response_wrapper(
            tasks.update,
        )


class AsyncTasksResourceWithStreamingResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.update = async_to_streamed_response_wrapper(
            tasks.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Question"]


class Question(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    prompt: str

    title: str

    type: Literal["categorical", "free_text"]
    """An enumeration."""

    choices: Optional[List[object]] = None
    """List of choices for the question. Required for CATEGORICAL questions."""

    conditions: Optional[List[object]] = None
    """Conditions for the question to be shown."""

    dropdown: Optional[bool] = None
    """Whether the question is displayed as a dropdown in the UI."""

    multi: Optional[bool] = None
    """Whether the question allows multiple answers."""

    required: Optional[bool] = None
    """Whether the question is required."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["S3DataSourceConfigParam"]


class S3DataSourceConfigParam(TypedDict, total=False):
    aws_account_id: Required[str]
    """AWS account ID that owns the S3 bucket."""

    aws_region: Required[str]
    """AWS region where the S3 bucket is located."""

    s3_bucket: Required[str]
    """Name of the S3 bucket where the data is stored."""

    source: Required[Literal["S3"]]

    s3_prefix: str
    """Prefix of the S3 bucket where the data is stored.

    If not specified, the entire bucket will be used.
    """
